@@ -8,28 +8,6 @@
  * file that was distributed with this source code.
  */
 
-// @codeCoverageIgnoreStart
-if (!defined('T_TRAIT')) {
-    define('T_TRAIT', 1001);
-}
-
-if (!defined('T_INSTEADOF')) {
-    define('T_INSTEADOF', 1002);
-}
-
-if (!defined('T_CALLABLE')) {
-    define('T_CALLABLE', 1003);
-}
-
-if (!defined('T_FINALLY')) {
-    define('T_FINALLY', 1004);
-}
-
-if (!defined('T_YIELD')) {
-    define('T_YIELD', 1005);
-}
-// @codeCoverageIgnoreEnd
-
 /**
  * Renders a PHP_CodeCoverage_Report_Node_File node.
  *
@@ -63,9 +41,7 @@ class PHP_CodeCoverage_Report_HTML_Renderer_File extends PHP_CodeCoverage_Report
 
         $this->htmlspecialcharsFlags = ENT_COMPAT;
 
-        if (PHP_VERSION_ID >= 50400 && defined('ENT_SUBSTITUTE')) {
-            $this->htmlspecialcharsFlags = $this->htmlspecialcharsFlags | ENT_HTML401 | ENT_SUBSTITUTE;
-        }
+        $this->htmlspecialcharsFlags = $this->htmlspecialcharsFlags | ENT_HTML401 | ENT_SUBSTITUTE;
     }
 
     /**
@@ -77,10 +53,10 @@ class PHP_CodeCoverage_Report_HTML_Renderer_File extends PHP_CodeCoverage_Report
         $template = new Text_Template($this->templatePath . 'file.html', '{{', '}}');
 
         $template->setVar(
-            array(
+            [
                 'items' => $this->renderItems($node),
                 'lines' => $this->renderSource($node)
-            )
+            ]
         );
 
         $this->setCommonTemplateVariables($template, $node);
@@ -104,7 +80,7 @@ class PHP_CodeCoverage_Report_HTML_Renderer_File extends PHP_CodeCoverage_Report
 
         $items = $this->renderItemTemplate(
             $template,
-            array(
+            [
                 'name'                         => 'Total',
                 'numClasses'                   => $node->getNumClassesAndTraits(),
                 'numTestedClasses'             => $node->getNumTestedClassesAndTraits(),
@@ -119,7 +95,7 @@ class PHP_CodeCoverage_Report_HTML_Renderer_File extends PHP_CodeCoverage_Report
                 'testedClassesPercent'         => $node->getTestedClassesAndTraitsPercent(false),
                 'testedClassesPercentAsString' => $node->getTestedClassesAndTraitsPercent(),
                 'crap'                         => '<abbr title="Change Risk Anti-Patterns (CRAP) Index">CRAP</abbr>'
-            )
+            ]
         );
 
         $items .= $this->renderFunctionItems(
@@ -168,7 +144,7 @@ class PHP_CodeCoverage_Report_HTML_Renderer_File extends PHP_CodeCoverage_Report
 
             $buffer .= $this->renderItemTemplate(
                 $template,
-                array(
+                [
                     'name'                         => $name,
                     'numClasses'                   => 1,
                     'numTestedClasses'             => $numTestedMethods == $numMethods ? 1 : 0,
@@ -207,7 +183,7 @@ class PHP_CodeCoverage_Report_HTML_Renderer_File extends PHP_CodeCoverage_Report
                         true
                     ),
                     'crap'                         => $item['crap']
-                )
+                ]
             );
 
             foreach ($item['methods'] as $method) {
@@ -255,7 +231,7 @@ class PHP_CodeCoverage_Report_HTML_Renderer_File extends PHP_CodeCoverage_Report
 
         return $this->renderItemTemplate(
             $template,
-            array(
+            [
                 'name'                         => sprintf(
                     '%s<a href="#%d"><abbr title="%s">%s</abbr></a>',
                     $indent,
@@ -288,7 +264,7 @@ class PHP_CodeCoverage_Report_HTML_Renderer_File extends PHP_CodeCoverage_Report
                     true
                 ),
                 'crap'                         => $item['crap']
-            )
+            ]
         );
     }
 
@@ -413,7 +389,7 @@ class PHP_CodeCoverage_Report_HTML_Renderer_File extends PHP_CodeCoverage_Report
     {
         $buffer              = file_get_contents($file);
         $tokens              = token_get_all($buffer);
-        $result              = array('');
+        $result              = [''];
         $i                   = 0;
         $stringFlag          = false;
         $fileEndsWithNewLine = substr($buffer, -1) == "\n";
@@ -439,11 +415,11 @@ class PHP_CodeCoverage_Report_HTML_Renderer_File extends PHP_CodeCoverage_Report
                 continue;
             }
 
-            list ($token, $value) = $token;
+            list($token, $value) = $token;
 
             $value = str_replace(
-                array("\t", ' '),
-                array('&nbsp;&nbsp;&nbsp;&nbsp;', '&nbsp;'),
+                ["\t", ' '],
+                ['&nbsp;&nbsp;&nbsp;&nbsp;', '&nbsp;'],
                 htmlspecialchars($value, $this->htmlspecialcharsFlags)
             );
 
